@@ -3,7 +3,7 @@
 
 #include <RcppEigen.h>
 
-#include "gpuR/eigen_helpers.hpp"
+#include "gpuR/dynEigenMat.hpp"
 #include "gpuR/cl_helpers.hpp"
 
 using namespace cl;
@@ -12,11 +12,12 @@ using namespace Rcpp;
 
 //[[Rcpp::export]]
 void cpp_gpuMatrix_igemm(SEXP ptrA_, SEXP ptrB_, SEXP ptrC_,
-    SEXP sourceCode_)
+    SEXP sourceCode_, int device_type)
 {
     // declarations
     cl_int err = 0;
     std::string sourceCode = as<std::string>(sourceCode_);
+    cl_device_type ocl_device;
     
     #if defined(__APPLE__) || defined(__MACOSX)
         #ifdef HAVE_OPENCL_CL2_HPP
@@ -30,17 +31,26 @@ void cpp_gpuMatrix_igemm(SEXP ptrA_, SEXP ptrB_, SEXP ptrC_,
         #endif
     #endif
     
-//    std::string kernel_string = as<std::string>(kernel_function_);
-//    const char* kernel_function = kernel_string.data();
-//    const char* kernel_function = (const char*)kernel_string.c_str();
-
-    Rcpp::XPtr<dynEigen<int> > ptrA(ptrA_);
-    Rcpp::XPtr<dynEigen<int> > ptrB(ptrB_);
-    Rcpp::XPtr<dynEigen<int> > ptrC(ptrC_);
+    XPtr<dynEigenMat<int> > ptrA(ptrA_);
+    XPtr<dynEigenMat<int> > ptrB(ptrB_);
+    XPtr<dynEigenMat<int> > ptrC(ptrC_);
     
-    Eigen::Map<Eigen::Matrix<int, Eigen::Dynamic, Eigen::Dynamic> > Am(ptrA->ptr(), ptrA->nrow(), ptrA->ncol());
-    Eigen::Map<Eigen::Matrix<int, Eigen::Dynamic, Eigen::Dynamic> > Bm(ptrB->ptr(), ptrB->nrow(), ptrB->ncol());
-    Eigen::Map<Eigen::Matrix<int, Eigen::Dynamic, Eigen::Dynamic> > Cm(ptrC->ptr(), ptrC->nrow(), ptrC->ncol());
+    Eigen::Ref<Eigen::Matrix<int, Eigen::Dynamic, Eigen::Dynamic> > refA = ptrA->data();
+    Eigen::Ref<Eigen::Matrix<int, Eigen::Dynamic, Eigen::Dynamic> > refB = ptrB->data();
+    Eigen::Ref<Eigen::Matrix<int, Eigen::Dynamic, Eigen::Dynamic> > refC = ptrC->data();
+    
+    Eigen::Map<Eigen::MatrixXi, 0, Eigen::OuterStride<> > Am(
+        refA.data(), refA.rows(), refA.cols(),
+        Eigen::OuterStride<>(refA.outerStride())
+    );
+    Eigen::Map<Eigen::MatrixXi, 0, Eigen::OuterStride<> > Bm(
+        refB.data(), refB.rows(), refB.cols(),
+        Eigen::OuterStride<>(refB.outerStride())
+    );
+    Eigen::Map<Eigen::MatrixXi, 0, Eigen::OuterStride<> > Cm(
+        refC.data(), refC.rows(), refC.cols(),
+        Eigen::OuterStride<>(refC.outerStride())
+    );
     
     int Mdim = Am.cols();
     int Ndim = Bm.rows();
@@ -62,7 +72,14 @@ void cpp_gpuMatrix_igemm(SEXP ptrA_, SEXP ptrB_, SEXP ptrC_,
         0
     };
 
-    Context context = createContext(CL_DEVICE_TYPE_GPU, cps, err);
+    // need to conditionally do CL_DEVICE_TYPE_CPU
+    if(device_type == 0){
+        ocl_device = CL_DEVICE_TYPE_GPU;
+    }else{
+        ocl_device = CL_DEVICE_TYPE_CPU;
+    }
+    
+    Context context = createContext(ocl_device, cps, err);
         
 //        std::cout << "Context Made" << std::endl;
 
@@ -144,8 +161,8 @@ void cpp_gpuMatrix_igemm(SEXP ptrA_, SEXP ptrB_, SEXP ptrC_,
     Buffer bufferC = Buffer(context, CL_MEM_WRITE_ONLY, szC * sizeof(int), NULL, &err);
 
     // Copy lists A and B to the memory buffers
-    queue.enqueueWriteBuffer(bufferA, CL_TRUE, 0, szA * sizeof(int), &Am(0));
-    queue.enqueueWriteBuffer(bufferB, CL_TRUE, 0, szB * sizeof(int), &Bm(0));
+    queue.enqueueWriteBuffer(bufferA, CL_TRUE, 0, szA * sizeof(int), Am.data());
+    queue.enqueueWriteBuffer(bufferB, CL_TRUE, 0, szB * sizeof(int), Bm.data());
 
         // Set arguments to kernel
 //        NDRange localWorkSize = NDRange(16, 16);
@@ -179,5 +196,5 @@ void cpp_gpuMatrix_igemm(SEXP ptrA_, SEXP ptrB_, SEXP ptrC_,
         
     // Read buffer C into a local list        
     err = queue.enqueueReadBuffer(bufferC, CL_TRUE, 0, 
-                                szC * sizeof(int), &Cm(0));
+                                szC * sizeof(int), Cm.data());
 }
